% assimilate multiple types of observational data (e.g. ocean tracers)
% and estimate posterior probability distribution of model parameters
run ~/startup.m
warning off
clear all
addpath MCMC

global param_type    %vector of parameters to be estimated
global opt_type      %select DE, MCMC1 (individual) and MCMC1 (joint)
global best_pos      %used to define vector "best" positions
global par_name      %name of the parameters
global vec           %vector containing broad parameters to be estimated  
global suffix
global corr_est

% initialize assimilation
[par,ipar,dx,pmin,pmax,obs_name,disp_name,disp_name2,par_name,par_val_name,disp_unit,homedir,datadir,direc,case1,corr_est] = initialize_mine;

% load observations and model outputs
[x obs obserr all_model_data ccoef best] = load_mine(obs_name,par_val_name,homedir,direc);

%If you want to plot the profiles, uncoment next line
plot_profiles

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Select type of optimization for the individual likelihoods
opt_type='MCMC1';%DE';%MCMC1'
[loglik best] =new_all_likelihoods(x,obs,obserr,par,all_model_data,best,ccoef,pmin,pmax);

%size(best)

save loglik.mat loglik best 
%Plot residuals and fittings
test_fit(obs_name,par,x,best,obs,all_model_data,par_val_name)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Select type of optimization for the joint likelihood
opt_type='MCMC2';

%Run new optimization with all tracers
[loglikat bestat] =new_all_likelihoods(x,obs,obserr,par,all_model_data,best,ccoef,pmin,pmax);

save loglikat.mat loglikat bestat 
%Add MOC pdf as a tracer for ploting.
%If you don't want to use comment next 2 lines
logL=MOC_Kv(par,ipar{1},par_val_name{1},loglikat,datadir,case1)
loglik(:,end+1,:)=logL; disp_name{end+1}='AMOC obs.';
%[logL]=MOC_Kv_24N(par{1},ipar);

%%%%%%%%%%%%%PLOTING SECTION%%%%%%%%%%%%%%%%%%%%%
%Plot individual tracers' pdfs
for i=1:length(par)
figure(i),clf
set(gcf,'PaperPosition',[1 1 5 4]);
set(gcf,'DefaultLineLineWidth',1.5)
wysiwyg;
[ilik ipost ilik_all ipost_all] = postprocess(loglik(:,:,i),par{i},ipar{i},dx(i));
%plot_pdfs(ipar{i},ipost,[],disp_name,par_name{i},[{'r' 'b'}],par{i}) 
plot_pdfs(ipar{i},ipost,[],disp_name,par_name{i},[],par{i}) 
hold on

%Add MOC pdf as a prior. Comment if logL is not specified above
%loglikat=[loglikat logL]; %disp_name2{end+1}={'AMOC obs.'};
%disp_name2={'all-tracers pdf','AMOC obs.'};
%
%Plot final pdf
figure(i)
[ilik ipost ilik_all ipost_all] = postprocess(loglikat(:,i),par{i},ipar{i},dx(i));
%
plot_pdfs(ipar{i},[],ipost_all,'Joint/all',par_name{i},{'-sk'},par{i});
%print('-depsc2',['figures/figure',num2str(i),'.ps'])
print('-depsc2',['figures/figure',num2str(i),suffix,'.ps'])
saveas(gcf, ['figures/figure',num2str(i),suffix,'.fig'], 'fig')
end

%Make MOC projection
%[ilik ipost ilik_all ipost_all] = postprocess(loglikat,par{1},ipar{1},dx(1));

moc_projection_1pannel(datadir,case1,loglikat,logL,par,ipar{1},[disp_name2 'Posterior']);
%[a]=moc_projection_1pannel_24N(ipost,ipost_all,par{1},ipar{1},[disp_name2 'Posterior']);
[ci1,ci_val1]=calc_95CI(loglik,par,ipar)
[ci2,ci_val2]=calc_95CI(loglikat,par,ipar)
save loglikat.mat ci1 ci_val1 ci2 ci_val2 -append 
