% log likelihood for multivariate normal distribution
% inspired by Tom Minka's Lightspeed Toolbox (normpdfln)
% Some other types added as well
% module: select betweeen svd,linear,chol,sep,AR1
%input:  x -> observations array;
%        mu -> model matrix;
%        Sigma -> auto-covariance matrix;
%        p     -> cross-covariance matrix;
function log_L = ldmvnorm(x,mu,Sigma,p,module)
N = length(x);

if nargin==4, module='svd0';end
%x=x(ones(1,size(mu,1)),:);

switch module
%%%%%%SVD ONLY - NON SEPARABLE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
case 'svd'
%if strcmp(module,'svd')
%isposdef = true;
%for i=1:N   %length(Sigma)
%  if ( det( Sigma(1:i, 1:i) ) <= 0 )
%    isposdef = false;
%    break;
%  end
%end
% if (~isposdef), 'not positive';log_L=-Inf;return;end

 [U,D,V] = svd(Sigma);
 detS    = prod(diag(D));
 iS      = V * diag(1./diag(D)) * U';
 log_L   = -0.5*N*log(2*pi) - 0.5*log(detS) - 0.5*(x-mu)*iS*(x-mu)';
 if isinf(log_L), log_L=-1000;end

%%%%%%SEPARABLE WITH SVD ONLY OR SVD/CHOL METHOD%%%%%%%%%%%%%%%%%%%%%%5
case 'sep'
%elseif strcmp(module,'sep')
 Np  =  length(p);
 NS  =  length(Sigma);
 [U,D,V] = svd(Sigma);
 detS = prod(diag(D));
 iS  =  V * diag(1./diag(D)) * U';

%CHOL FOR CORRELATION%%%%%%%%
 [Rp,c2] = chol(p);
 if (c2 ~= 0), 'not positive';log_L=-Inf;return;end
 ip  =  Rp\eye(Np); ip = ip * ip';
 logdetip = -sum(log(diag(Rp))); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%SVD FOR CORRELATION%%%%%%%%%
% [U,D,V] = svd(p);
% detp = prod(diag(D));
% ip = V * diag(1./diag(D)) * U';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 Sig_inv = kron(ip,iS);
%%%%% log_L = -0.5*N*log(2*pi) - 0.5*log(detS) ...
%THIS IS IF I USE SVD FOR CORRELATION
% log_L = -0.5*N*log(2*pi) - 0.5*Np*log(detS) - 0.5*NS*log(detp) ...
%THIS IS IF I USE CHOL FOR CORRELATION
 log_L = -0.5*N*log(2*pi) - 0.5*Np*log(detS) + NS*logdetip ...
          - 0.5*(x-mu)*Sig_inv*(x-mu)';
 if isinf(log_L),'infinite', log_L=-10000;end

%%%%%%SEPARABLE WITH CHOL ONLY METHOD%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% slower but more numerically stable singular value decomposition to
% replace Cholesky
case 'chol'
%elseif strcmp(module,'chol')
  Np  =  length(p);
  NS  =  length(Sigma);
 [R,c]= chol(Sigma); % upper Cholesky factor of covariance matrix
 [Rp,c2]=chol(p);
 if (c~=0 | c2~=0), 'not positive';log_L=-Inf;return;end

 dx = (x-mu)'; % residual: subtract mean
 iS = sparse(R\eye(NS)); 
 ip = sparse(Rp\eye(Np)); 
 Sig_inv=kron(ip,iS);
 dx = Sig_inv'*dx; % to compute dx^T S^-1 dx faster
% xa = reshape(dx,NS,Np);

 % -log(det(R)) = -1/2 log(det(Sigma)) = -log(sqrt(det(Sigma)))
 logdetiR = -sum(log(diag(R))); 
 logdetip = -sum(log(diag(Rp))); 

 % multivariate normal
% log_L = (logdetiR - 0.5*N*log(2*pi)) - 0.5*(dx'*dx); 
 log_L = -0.5*N*log(2*pi) + Np*logdetiR + NS*logdetip ...
          - 0.5*(dx'*dx);
%      - 0.5* trace((xa*ip)'*(iS*xa));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 'AR1'
%elseif strcmp(module,'AR1')
%Solves the Toplitz matrix analiticaly
   Np = length(p);
   NS = length(Sigma);
  rho = Sigma(end,end-1);
 detS = (1-rho^2)^(NS-1);
   iS = eye(NS,NS) * (1+rho^2);
   iS(2:NS,:) = iS(2:NS,:) + (iS(1:NS-1,:)*-rho);
 iS(1:NS-1,:) = (iS(1:NS-1,:) + (iS(2:NS,:)*-rho));
 iS = iS./(1-rho^4);
%Chol decomposition
 [Rp,c2] = chol(p);
 if (c2 ~= 0), 'not positive';log_L=-Inf;return;end
 ip  =  Rp\eye(Np); ip = ip * ip';
 logdetip = -sum(log(diag(Rp)));

% dx = (x-mu)' ;
% xa = reshape(dx,NS,Np);


 % log_L = -0.5*N*log(2*pi) - 0.5*log(detS) - 0.5*(x-mu)*iS*(x-mu)';
 log_L = -0.5*N*log(2*pi) - 0.5*Np*log(detS) + NS*logdetip ...
          - 0.5*(x-mu)*kron(ip,iS)*(x-mu)';
%      - 0.5* trace((xa*ip)'*(iS*xa));
 if isinf(log_L), log_L=-1000;end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 'linear'
%elseif strcmp(module,'linear')
%%%Solves linear system for the separable matrix
%new method ; very fast
 Np = length(p);
 NS = length(Sigma);
 [Rp,c2]=chol(p);
 if (c2~=0), 'not positive';log_L=-Inf;return;end
 dx = (x-mu)' ;
 xa = reshape(dx,NS,Np);
 ip = sparse(Rp)\eye(Np); ipx = xa * (ip*ip');
 logdetip = -sum(log(diag(Rp))); 
 lower.LT = true ;
 upper.UT = true ;
 [L,U,perm] = lu (Sigma, 'vector') ;
  detS = abs(prod(diag(U)));
   dx2 = (linsolve (U, linsolve (L, xa(perm,:), lower), upper)) ;
 log_L = -0.5*N*log(2*pi) - 0.5*Np*log(detS) + NS*logdetip ...
  - 0.5*trace(ipx'*dx2);
%trace((xa*ip)'*(iS*xa))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
otherwise
%else
%%%Solves linear system for the full covariance matrix S
%solves Ax=b
 [Rp,c2] = chol(p);
 if (c2~=0), 'not positive';log_L=-Inf;return;end
  S = kron(p,Sigma);
        dx = (x-mu)' ;
  lower.LT = true ;
  upper.UT = true ;
[L,U,perm] = lu (S, 'vector') ; 
 dx2 = (linsolve (U, linsolve (L, dx(perm), lower), upper)) ;
 dx  = dx'*dx2;
detS = abs(prod(diag(U)));
%%norm (A*x-b)

 log_L = -0.5*N*log(2*pi) - 0.5*log(detS) - 0.5*dx;
% log_L = diag(log_L);
 if isinf(log_L), log_L=-1000;end

end
