% evaluate Gaussian likelihood function for a given set of observations and model run
% Can fix statistical parameters at best estimates if wanted
%
% optimize statistical hyperparameters to maximize posterior probability
% the (log) likelihood function is evaluated in 'multi_obj'
% as are the statistical parameter priors

% since we are applying a prior to the statistical parameters (although not
% the model parameters) in the objective function, this isn't really a pure
% likelihood function
% the variable opt_type selects 'DE' or 'MCMC'

function [log_likelihood best_hyperparams] = multi_likelihood(x,model,obs,obserr,...
    modelpar,ccoef,pmin,pmax,best,param_type)

global opt_type par_name
%dbstop('27')
% parameters to pass to objective function (needed to compute posterior)
args.x=x;               args.model=model;
args.obs=obs;           args.obserr=obserr;
args.modelpar=modelpar; args.param_type=param_type;
args.best=best;         args.ccoef=ccoef;


if strcmp(opt_type,'DE')
% initialize DE optimization parameters
n_params = length(pmin);
VTR = -Inf; popsize = 40; maxiter = 500; %FOR DE 
% differential evolution (DE) global optimization algorithm
[best_hyperparams best_obj] = devec3('multi_obj',VTR,n_params,pmin,pmax,args,popsize,maxiter);

else

% initialize MCMC optimization parameters
maxiter = 20000;  %FOR MCMC
%REAL MCMC method
[best_hyperparams best_obj] = roda_metro('multi_obj',par_name,pmin,pmax,args,maxiter);
best_obj=-log(-best_obj);

end

% posterior probability of this model run (set of model parameters)
log_likelihood = -best_obj;
